-- Broken Bottle - 7DRL
-- Copyright (C) 2011 Darren Grey
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.


require "engine.class"
local DamageType = require "engine.DamageType"
local Map = require "engine.Map"
local Target = require "engine.Target"
local Talents = require "engine.interface.ActorTalents"
local Chat = require "engine.Chat"

--- Interface to add ToME combat system
module(..., package.seeall, class.make)

--- Checks what to do with the target
-- Talk ? attack ? displace ?
function _M:bumpInto(target)
	local reaction = self:reactionToward(target)
	if reaction < 0 then
		-- piranha stealing
      if self.subtype == "child" then
         self.ai = "flee_dmap"
         local inven = target:getInven("INVEN")
         if #inven >= 1 then
            local o = target:removeObject(inven, rng.range(1, #inven))
            local desc = self.name
            if target.inebriation > 60 then desc = self.drunk_name end
            game.log("The %s grabs at me and steals my %s.",desc,o.name)
            self:addObject(self:getInven("INVEN"), o)
         end
      else return self:attackTarget(target) end
	elseif reaction >= 0 then
		-- Talk ?
		if self.player and target.can_talk then
			local chat = Chat.new(target.can_talk, target, self)
			chat:invoke()
			if target.can_talk_only_once then target.can_talk = nil end
		elseif target.player and self.can_talk then
			local chat = Chat.new(self.can_talk, self, target)
			chat:invoke()
			if target.can_talk_only_once then target.can_talk = nil end
		elseif self.move_others and not target.cant_be_moved then
			if target.move_others and self ~= game.player then return end

			-- Displace
			local tx, ty, sx, sy = target.x, target.y, self.x, self.y
			target.x = nil target.y = nil
			self.x = nil self.y = nil
			target:move(sx, sy, true)
			self:move(tx, ty, true)
		end
	end
end

--- Makes the death happen!
function _M:attackTarget(target, mult)
	if target.subtype == "child" and self.inebriation < 60 then
		local message = rng.table{"I look into the kid's eyes.  I can't... I can't do it.","I raise my hand to attack, but... no, no I won't...","I done enough bad things in life, I don't want to kill a child.","I ain't attackin' no damned kid!"}
		game.logPlayer(self, "%s", message)
	else
				
		if self.combat then
		  local tohit = 0
		  if self.stamina < self.stamina_cost then
			 tohit = rng.range(0,(2*self:getAim()*(self.stamina+1)/(self.stamina_cost+1)))
		  else tohit = rng.range(0,(2*self:getAim()))
		  end
		  
		  def = rng.range(0,target:getSpd())
		  
		  if tohit < def then
            if self.stamina > self.stamina_cost then game.logPlayer(self, "I swing and miss.") else game.logPlayer(self, "My tired swing completely misses.") end
            if target == game.player then
               local desc = self.name
               if target.inebriation > 60 then desc = self.drunk_name end
               game.log("The %s misses me.",desc)
            end
		  else
                        
            local critchance = rng.range(0,(0.5*self:getAim() + self.combat.crit_chance))
            local critdef = rng.range(0,(target:getSpd() + target.crit_defence))
            if critchance > critdef then local crit = true end
            
            local dam = self.combat.dam + (self:getStr() / 3) - target.combat_armor
            if dam < 1 then dam = 1 end
            if crit then dam = dam + 0.5*self:getAim() end
            
            DamageType:get(DamageType.PHYSICAL).projector(self, target.x, target.y, DamageType.PHYSICAL, math.max(0, dam))
            
            self.stamina = self.stamina - self.stamina_cost -- Melee consumes stamina
            if self.stamina < 0 then self.stamina = 0 end
		  end
      end

      if not no_actions then
		self:useActionPoints(20)
		core.game.sleep(50)
	end
    end
end
