-- Broken Bottle - 7DRL
-- Copyright (C) 2011 Darren Grey
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even th+e implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.


require "engine.class"
require "mod.class.Actor"

local Dialog = require "engine.ui.Dialog"
local ListColumns = require "engine.ui.ListColumns"
local Textzone = require "engine.ui.Textzone"
local TextzoneList = require "engine.ui.TextzoneList"
local Separator = require "engine.ui.Separator"


module(..., package.seeall, class.inherit(Dialog))

local _points_text = "Stats points left: #00FF00#%d#WHITE#"

function _M:init(actor)
	self.actor = actor
	self.actor_dup = actor:clone()
	self.unused_stats = self.actor.unused_stats
	Dialog.init(self, "Stats Levelup: "..actor.name, 600, 500)

	self.sel = 1

	self.c_tut = Textzone.new{width=math.floor(self.iw / 2 - 10), auto_height=true, no_color_bleed=true, text=[[
Keyboard: #00FF00#up key/down key#FFFFFF# to select a stat; #00FF00#right key#FFFFFF# to increase stat; #00FF00#left key#FFFFFF# to decrease a stat.
Mouse: #00FF00#Left click#FFFFFF# to increase a stat; #00FF00#right click#FFFFFF# to decrease a stat.
]]}
	self.c_desc = TextzoneList.new{width=math.floor(self.iw / 2 - 10), height=self.ih - self.c_tut.h - 20, no_color_bleed=true}
	self.c_points = Textzone.new{width=math.floor(self.iw / 2 - 10), auto_height=true, no_color_bleed=true, text=_points_text:format(self.actor.unused_stats)}

	self.c_list = ListColumns.new{width=math.floor(self.iw / 2 - 10), height=self.ih - 10, all_clicks=true, columns={
		{name="Stat", width=50, display_prop="name"},
		{name="Base", width=25, display_prop="base"},
		{name="Value", width=25, display_prop="val"},
	}, list={
		{name="Strength",     base=self.actor:getStr(nil, nil, true), val=self.actor:getStr(), stat_id=self.actor.STAT_STR},
		{name="Aim",    base=self.actor:getAim(nil, nil, true), val=self.actor:getAim(), stat_id=self.actor.STAT_AIM},
		{name="Health",        base=self.actor:getHea(nil, nil, true), val=self.actor:getHea(), stat_id=self.actor.STAT_HEA},
		{name="Speed",    base=self.actor:getSpd(nil, nil, true), val=self.actor:getSpd(), stat_id=self.actor.STAT_SPD},
	}, fct=function(item, _, v)
		self:incStat(v == "left" and 1 or -1)
	end, select=function(item, sel) self.sel = sel self.c_desc:switchItem(item, self.actor.stats_def[item.stat_id].description) end}

	self:loadUI{
		{left=0, top=0, ui=self.c_points},
		{left=5, top=self.c_points.h+5, ui=Separator.new{dir="vertical", size=math.floor(self.iw / 2) - 10}},
		{left=0, top=self.c_points.h+15, ui=self.c_list},

		{hcenter=0, top=5, ui=Separator.new{dir="horizontal", size=self.ih - 10}},

		{right=0, top=self.c_tut.h + 20, ui=self.c_desc},
		{right=0, top=0, ui=self.c_tut},
	}
	self:setFocus(self.c_list)
	self:setupUI()

	self:update()

	self.key:addBinds{
		EXIT = function()
         game:unregisterDialog(self)
         self:finish()
		end,
	}
end

function _M:finish()
	if self.actor.unused_stats == self.unused_stats then return end
	local reset = {}
	
end

function _M:incStat(v)
	if v == 1 then
		if self.actor.unused_stats <= 0 then
			self:simplePopup("Not enough stat points", "No stat points left.")
			return
		end
	else
		if self.actor_dup:getStat(self.sel, nil, nil, true) == self.actor:getStat(self.sel, nil, nil, true) then
			self:simplePopup("Impossible", "Cannot take out more points.")
			return
		end
	end

	local sel = self.sel
	self.actor:incStat(sel, v)
	self.actor.unused_stats = self.actor.unused_stats - v
	self.c_list.list[sel].base = self.actor:getStat(sel, nil, nil, true)
	self.c_list.list[sel].val = self.actor:getStat(sel)
	self.c_list:generate()
	self.c_list.sel = sel
	self.c_list:onSelect()
	self.c_points.text = _points_text:format(self.actor.unused_stats)
	self.c_points:generate()
   self.actor.max_life = 25 + 2*(self.actor.level - 1) + 2*(self.actor:getHea() - 10)
   self.actor.max_stamina = 55 + 5*self.actor.level + 3*self.actor:getStr() + self.actor:getHea()
   local speed = 1 + 0.02*(self.actor:getSpd())
   self.actor.energy.mod = speed
   self.actor.life = self.actor.max_life
   self.actor.stamina = self.actor.max_stamina
	self:update()
end

function _M:update()
	self.c_list.key:addBinds{
		ACCEPT = function() self.key:triggerVirtual("EXIT") end,
		MOVE_LEFT = function() self:incStat(-1) end,
		MOVE_RIGHT = function() self:incStat(1) end,
	}
end


function _M:drawDialog(s)
	-- Description part
	self:drawHBorder(s, self.iw / 2, 2, self.ih - 4)
	local statshelp = ([[Keyboard: #00FF00#up key/down key#FFFFFF# to select a stat; #00FF00#right key#FFFFFF# to increase stat; #00FF00#left key#FFFFFF# to decrease a stat.
Mouse: #00FF00#Left click#FFFFFF# to increase a stat; #00FF00#right click#FFFFFF# to decrease a stat.
]]):splitLines(self.iw / 2 - 10, self.font)
	local lines = self.actor.stats_def[self.sel].description:splitLines(self.iw / 2 - 10, self.font)
	for i = 1, #statshelp do
		s:drawColorStringBlended(self.font, statshelp[i], self.iw / 2 + 5, 2 + (i-1) * self.font:lineSkip())
	end
	for i = 1, #lines do
		s:drawColorStringBlended(self.font, lines[i], self.iw / 2 + 5, 2 + (i + #statshelp + 1) * self.font:lineSkip())
	end

	-- Stats
	s:drawColorStringBlended(self.font, "Stats points left: #00FF00#"..self.actor.unused_stats, 2, 2)
	self:drawWBorder(s, 2, 20, 200)

	self:drawSelectionList(s, 2, 25, self.font_h, {
		"Strength", "Aim", "Health", "Speed"
	}, self.sel)
	self:drawSelectionList(s, 100, 25, self.font_h, {
		self.actor:getStr(), self.actor:getAim(), self.actor:getHea(), self.actor:getSpd(),
	}, self.sel)
	self.changed = false
end
