-- Hellfire
-- Copyright (C) 2011 Sean "Chezzo" Osman sean@chezzo.com
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.




-- This file loads the game module, and loads data
local KeyBind = require "engine.KeyBind"
local DamageType = require "engine.DamageType"
local ActorStats = require "engine.interface.ActorStats"
local ActorResource = require "engine.interface.ActorResource"
local ActorTalents = require "engine.interface.ActorTalents"
local ActorAI = require "engine.interface.ActorAI"
local ActorLevel = require "engine.interface.ActorLevel"
local ActorTemporaryEffects = require "engine.interface.ActorTemporaryEffects"
local Birther = require "engine.Birther"
local ActorInventory = require "engine.interface.ActorInventory"
local Map = require "engine.Map"

-- Useful keybinds
KeyBind:load("move,hotkeys,inventory,actions,interface,debug")

KeyBind:defineAction{
   default = { "sym:_s:false:false:false:false" },
   type = "SHOOT",
   group = "actions",
   name = "Shoot",
}

KeyBind:defineAction{
   default = { "sym:_COMMA:false:true:false:false", "mouse:right:false:false:false:false" }, 
   type = "CHANGE_LEVEL",
   group = "actions",
   name = "Change Level",
}

--KeyBind:loadRemap("/keybinds2.cfg")


-- Additional entities resolvers
dofile("/mod/resolvers.lua")

-- Damage types
DamageType:loadDefinition("/data/damage_types.lua")

-- Talents
ActorTalents:loadDefinition("/data/talents.lua")

-- Timed Effects
ActorTemporaryEffects:loadDefinition("/data/timed_effects.lua")

-- Actor resources
ActorResource:defineResource("Life", "life", nil, "life_regen", "Life is how hurt I currently am.")
ActorResource:defineResource("Stamina", "stamina", nil, "stamina_regen", "Stamina is needed to use weapons in melee.")
ActorResource:defineResource("Action Points", "actions", nil, "actions_regen", "Action points represent how much I can move or attack each turn.")

-- Actor stats
ActorStats:defineStat("Strength",	"str", 10, 1, 100, "Strength lets me hit harder against whatever comes against me, and builds up my stamina.\n1 Strength = +0.33 damage, +3 stamina")
ActorStats:defineStat("Aim",	"aim", 10, 1, 100, "Aim gives me a better chance to hit, and to target vital spots.\n1 Aim = +2% hit chance, +1% crit chance, +0.5 crit damage")
ActorStats:defineStat("Health",	"hea", 10, 1, 100, "Health allows me to cling on to life a little longer in this desperate world.\n1 Health = +2 Life, +1 Stamina")
ActorStats:defineStat("Speed",	"spd", 10, 1, 100, "Speed lets me move faster, and dodge attacks easier.\n1 Speed = +2% speed, +1% defence, +2% crit defence")

-- Actor AIs
ActorAI:loadDefinition("/engine/ai/")
ActorAI:loadDefinition("/mod/ai/")

-- Birther descriptor
Birther:loadDefinition("/data/birth/descriptors.lua")

-- Equipment
ActorInventory:defineInventory("MELEE", "melee weapon", true, "Weapons used for hand to hand combat")
ActorInventory:defineInventory("RANGED", "ranged weapon", true, "Weapons to fire or throw at a distance")
ActorInventory:defineInventory("HEAD", "headgear", true, "Protective gear for the head - helps against headshots")
ActorInventory:defineInventory("BODY", "body armour", true, "Protective gear for the body")

-- For smooth movement
Map.smooth_scroll = 6

-- Enable hex mode
core.fov.set_algorithm("hex")

return {require "mod.class.Game" }
